#!/usr/bin/env python2
################################################
#
#   by Joh Schoeneberg
#   www.schoeneberglab.org
#   Nov 2014
#
################################################
# classes

# this class is a topology unit object
# it stores topology kind : 'rig' or 'flx'
# domainId, start and end residue of the topology

import argparse
import os
import operator

def parseArguments():
    parser = argparse.ArgumentParser(
        description="""Select structures for analysis

        the script uses the weights.dat file from fitting
        to determine the most important clusters, 
        determines the cluster center structure of these 
        clusters and eventually selects them in the 
        ensemble and outputs them in a single pdb file.
        """, formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('weights_filename', type=str,
                        help='weighs.dat')
    parser.add_argument('clusters_filename', type=str,
                        help='clusters.dat')
    parser.add_argument('ensemble_filename', type=str,
                        help='chmp3_ensemble_01.pdb')
    parser.add_argument('--output', '-o', const=1, nargs='?', type=str,
                        default='fit_result_structures.pdb',
                        help='file to store the fit result pdb structures')
    parser.add_argument('--nClust', type=int,
                        help='Number of clusters considered')

    args = parser.parse_args()
    return args

# this class contains a list of all lines within a pdb structure
class PdbStructure:

    def __init__(self, listOfLines):
        self.listOfLines = listOfLines

    def printout(self):
        for line in self.listOfLines:
            print(line)


def read_weightsFile(weights_filename):
    clusterId2weight = dict()
    print('read weights from: '+os.path.abspath(weights_filename))
    with open(weights_filename, 'r') as weigthsFile:
        for line in weigthsFile:
            lineParts = line.split()
            #clusterId, w0, w, alpha, chi^2, G
            clusterId = int(lineParts[0])
            w = float(lineParts[2])
            print( 'clusterId: '+str(clusterId)+ " weight: " + str(w) )
            clusterId2weight[clusterId] = w
            
    return clusterId2weight
    
def read_clustersFile(clusters_filename):
    clusterId2centerStructureId = dict()
    print('read clusters from: '+os.path.abspath(clusters_filename))
    with open(clusters_filename, 'r') as clustersFile:
        currentClusterId = -1
        for line in clustersFile:
            lineParts = line.split()
            #clusterId, StructureID, clusterCenterStructureId, nStructuresPerCluster
            clusterId = int(lineParts[0])
            StructureID = int(lineParts[1])
            clusterCenterStructureId = int(lineParts[2])
            nStructuresPerCluster = int(lineParts[3])
            if(clusterId != currentClusterId):
                currentClusterId = clusterId
                clusterId2centerStructureId[currentClusterId] = clusterCenterStructureId                
                print( 'clusterId: '+str(clusterId)+ " clusterCenterStructureId: " + str(clusterCenterStructureId))
            
    return clusterId2centerStructureId
    
    
#def read_inputPdbFile(inputStructure_filename):
#    pdbFileLines = list()
#    print('read pdb file from: '+os.path.abspath(inputStructure_filename))
#    with open(inputStructure_filename, 'r') as inputPdbFile:
#        for line in inputPdbFile:
#            pdbFileLines.append(line)
#    return pdbFileLines
    
    
def read_ensemblePdbFile(list_relevantStructureIds, ensemble_filename):
    structureId2PdbStructure = dict()
    print('read pdb ensemble file from: '+os.path.abspath(ensemble_filename))
    with open(ensemble_filename, 'r') as inputPdbEnsembleFile:
        currentlyRecording = False
        currentStructureId = -1
        currentStructureLines = list()
        
        for line in inputPdbEnsembleFile:
            lineParts = line.split()

            
            # check if we have found a relevant structure
            if(lineParts[0]=="MODEL"):
                currentStructureId = int(lineParts[1])
                #print("checking id "+str(currentStructureId))
                for lookedForStructureId in list_relevantStructureIds:
                    #print ("relevant structure ids "+str(lookedForStructureId))
                    if(lookedForStructureId == currentStructureId):
                        # ok, its a hit
                        currentlyRecording = True
                        print("record MODEL "+str(currentStructureId))
                        
            # append the current line if we are recording
            if(currentlyRecording):
                currentStructureLines.append(line)
                #print("record line")
                #for action in currentStructureLines:
                #    print(action)

            # check if we might have to stop recording 
            if(lineParts[0]=="ENDMDL" and currentlyRecording):
                currentlyRecording = False
                # append the recorded lines to the collection list
                currentPdbStructure = PdbStructure(currentStructureLines)
                structureId2PdbStructure[currentStructureId] = currentPdbStructure
                #for action in currentStructureLines:
                #    print(action)
                # reset line container
                currentStructureLines = list()
                
    return structureId2PdbStructure    
            


def main():
    args = parseArguments()
    # assign args variables
    weights_filename = args.weights_filename
    clusters_filename = args.clusters_filename
    ensemble_filename = args.ensemble_filename
    output_filename = args.output
    nClust = args.nClust
    

    clusterId2weight = read_weightsFile(weights_filename)
    clusterId2centerStructureId = read_clustersFile(clusters_filename)

    # here we collect the cluster center structures from the ensemble
    structureId2PdbStructure = read_ensemblePdbFile(clusterId2centerStructureId.values(), ensemble_filename)
    
    # output
    with open(output_filename, 'w') as outputFile:
        outputCounter = 0
        # sort the clusterId by the reweighted weight, biggest to smallest weight
        for clusterId_weight_tuple in reversed(sorted(clusterId2weight.items(), key=operator.itemgetter(1))):
            print("clusterId "+str(clusterId_weight_tuple))
            clusterId = clusterId_weight_tuple[0]
            if(outputCounter < nClust):
                # write into the output file
                structureId = clusterId2centerStructureId[clusterId]
                clusterCenterPdbStructure = structureId2PdbStructure[structureId]
                for line in clusterCenterPdbStructure.listOfLines:
                     outputFile.writelines(line)
                print("structure "+str(clusterId)+ " written.")
                outputCounter = outputCounter +1
            else:
                print("no further output, nClust="+str(nClust)+" to write already reached")
                
    #done
    print("Done.")                
    
        
    
    
if __name__ == '__main__':
    main()